<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Handler_Map extends Controller_Handler
{
    /** @var  Model_User_Teacher */
    protected $user;


    public function before() {
        parent::before();

        $this->user->checkAccess(User::RIGHTS_TEACHER);
    }


    public function action_ChangeDisciplineSubject() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->ChangeDisciplineSubject($this->user->TeacherID, $_POST['SubjectID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeStatusBonusModule() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post['BonusRate'] == true) {
            $failed = $map->AddModuleBonus($this->user->TeacherID);
            $response['action'] = 'add';
        } else {
            $failed = $map->DeleteModuleBonus($this->user->TeacherID);
            $response['action'] = 'delete';
        }

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeDisciplineGrade() {
        $failed = Model_Discipline::load($_POST['DisciplineID'])
            ->changeGrade($this->user->TeacherID, $_POST['GradeID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeDisciplineControl() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $result = $map->ChangeDisciplineControl($this->user->TeacherID, $_POST['Control']);

        $response['success'] = $result;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeDisciplineHours() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->ChangeDisciplineHours($this->user->TeacherID, $_POST['Hours'], $_POST['Type']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_GetSubjectsList() {
        $faculty = Model_Faculty::with($_POST['FacultyID']);
        $subjectsList = $faculty->getSubjects();

        $this->response->body(json_encode($subjectsList));
    }

    public function action_AddModule() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $response['moduleID'] = $map->addModule($this->user->TeacherID);
        if ($response['moduleID'] > 0) {
            // Module must contain 1 submodule at least
            $response['submoduleID'] = $map->addSubmodule($this->user->TeacherID, $response['moduleID'], 'CurrentControl');

            if ($response['submoduleID'] > 0)
                $response['success'] = true;
        }

        $this->response->body(json_encode($response));
    }

    public function action_AddSubmodule() {
        $response['submoduleID'] = Model_Map::addSubmodule($this->user->TeacherID, $_POST['submoduleID'], 'CurrentControl');
        if ($response['submoduleID'] > 0)
            $response['success'] = true;

        $this->response->body(json_encode($response));
    }

    public function action_ChangeModuleName() {
        Model_Map::changeModuleName($this->user->TeacherID, $_POST['ModuleID'], $_POST['ModuleName']);

        $response['success'] = true;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeSubmoduleName() {
        $failed = Model_Map::changeSubmoduleName($this->user->TeacherID, $_POST['SubmoduleID'], $_POST['SubmoduleName']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_SwapModuleOrder() {
        $failed = Model_Map::swapModuleOrder($this->user->TeacherID, $_POST['ModuleID1'], $_POST['ModuleID2']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_SwapSubmoduleOrder() {
        $failed = Model_Map::swapSubmoduleOrder($this->user->TeacherID, $_POST['SubmoduleID1'], $_POST['SubmoduleID2']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_DeleteModule() {
        $failed = Model_Map::deleteModule($this->user->TeacherID, $_POST['ModuleID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_DeleteSubmodule() {
        $failed = Model_Map::deleteSubmodule($this->user->TeacherID, $_POST['SubmoduleID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeSubmoduleMaxAndControl() {
        $failed = Model_Map::changeSubmoduleMaxAndControl(
            $this->user->TeacherID, $_POST['SubmoduleID'], $_POST['MaxRate'], $_POST['ControlType']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_BindGroup() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $code = $map->bindGroup($this->user->TeacherID, $_POST['GroupID']);

        $response = ['success' => (0 == $code), 'code' => $code,];
        $this->response->body(json_encode($response));
    }

    public function action_UnbindGroup() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->unbindGroup($this->user->TeacherID, $_POST['GroupID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_BindStudent() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->BindStudent($this->user->TeacherID, $_POST['StudentID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_UnbindStudent() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->UnbindStudent($this->user->TeacherID, $_POST['StudentID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_GetGroups() {
        $faculty = Model_Faculty::with($_POST['FacultyID']);
        $groups = $faculty->getGroups($_POST['GradeID']);
        $this->response->body(json_encode($groups));
    }

    public function action_SearchStudents() {
        $result = Model_Students::search($_POST['GradeID'], $_POST['GroupID'], $_POST['FacultyID'], $_POST['Name'], $_POST['DisciplineID']);
        $this->response->body(json_encode($result));
    }

    public function action_GetDepartments() {
        $faculty = Model_Faculty::with($this->post['FacultyID']);
        $departments = $faculty->getDepartments();
        $this->response->body(json_encode($departments));
    }

    public function action_SearchTeachers() {
        $teachers = Model_Teachers::search($_POST['Last'], $_POST['FacultyID'], $_POST['DepartmentID'], $_POST['DisciplineID']);
        $this->response->body(json_encode($teachers));
    }
}
