<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Student_Subject extends Controller_Environment_Student
{
    public function action_show() {
        $id = $this->request->param('id');
        $discipline = Model_Discipline::load($id);
        $rates = Model_Rating::getRates($id, $this->user->StudentID);

        $this->twig->set([
            'Discipline'    => $discipline,
            'Teachers'      => self::getTeachersListOf($discipline),
            'DisciplineMap' => self::generateDisciplineMapFrom($rates),
            'Semester'      => Model_Semesters::load($discipline->SemesterID),
        ])->set_filename(static::STUDENT . 'subject');
    }

    private static function &getTeachersListOf(Model_Discipline $discipline) {
        $teachers = $discipline->getTeachers()->as_array();

        foreach ($teachers as &$t) {
            $name = [ $t['LastName'], $t['FirstName'], $t['SecondName'] ];
            $t['Name'] = trim(implode($name, ' '));
        }

        return $teachers;
    }

    private static function generateDisciplineMapFrom(Database_Result $rates) {
        $id = 0;                   # not sure, what it is about :(
        $sRate = 0;                # completely legacy code, baby
        $sMaxRate = 0;
        $extraRate = 0;
        $bonusRate = 0;
        $bonusMaxRate = 0;
        $examRate = 0;
        $examMaxRate = 0;
        $disMap = array();
        $modulesCount = 0;
        $disMap['Extra']['Rate'] = 0;

        foreach ($rates as $row) {
            switch ($row['ModuleType']) {
                case 'exam':
                    $examRate += $row['Rate'];
                    $examMaxRate += (int)$row['MaxRate'];

                    $disMap['Exam'] = array(
                        'Rate'    => (int)$row['Rate'],
                        'MaxRate' => (int)$row['MaxRate'],
                        'Date'    => $row['Date'],
                    );
                    break;

                case 'bonus':
                    $bonusRate += $row['Rate'];
                    $bonusMaxRate += (int)$row['MaxRate'];

                    $disMap['Bonus'] = array(
                        'Description' => $row['Description'],
                        'Rate'        => (int)$row['Rate'],
                        'MaxRate'     => (int)$row['MaxRate'],
                        'Date'        => $row['Date'],
                    );
                    break;

                case 'extra':
                    $extraRate += (int)$row['Rate'];

                    $disMap['Extra']['Description'] = $row['Description'];
                    $disMap['Extra']['Rate'] += (int)$row['Rate'];
                    break;

                default:
                    $sRate += (int)$row['Rate'];
                    $sMaxRate += (int)$row['MaxRate'];

                    if ($row['ModuleID'] != $id) {
                        $id = $row['ModuleID'];
                        $modulesCount++;
                    }

                    $module =& $disMap['Modules'][$modulesCount];

                    if (!isset($module['SubmodulesCount'])) {
                        $module['Rate'] = 0;
                        $module['MaxRate'] = 0;
                        $module['SubmodulesCount'] = 0;
                    }

                    $module['Title']    = $row['ModuleName'];
                    $module['Rate']     += (int)$row['Rate'];
                    $module['MaxRate']  += (int)$row['MaxRate'];
                    $module['SubmodulesCount']++;

                    $module['Submodules'][] = array(
                        'Date'        => $row['Date'],
                        'Title'       => $row['SubmoduleName'],
                        'Description' => $row['SubmoduleDescription'],
                        'Rate'        => (int)$row['Rate'],
                        'MaxRate'     => (int)$row['MaxRate'],
                    );
            }
        }
        $disMap['ModulesCount'] = $modulesCount;

        $disMap['SemesterRate'] = (int)$sRate;
        $disMap['SemesterMaxRate'] = (int)$sMaxRate;
        $disMap['ExamRate'] = (int)$examRate;
        $disMap['ExamMaxRate'] = (int)$examMaxRate;
        $disMap['BonusRate'] = (int)$bonusRate;
        $disMap['BonusMaxRate'] = (int)$bonusMaxRate;
        $disMap['ExtraRate'] = (int)$extraRate;

        return $disMap;
    }
}